///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

#ifdef __linux__
# include <unistd.h>
#endif

#include <wx/dir.h>
#include <wx/colordlg.h>

#include "EditorFrame.h"
#include "EditorGLCanvas.h" 
#include "LayoutGLCanvas.h" 
#include "DialogResourceGroups.h"

// CEGUI includes
#include <CEGUIWindowFactoryManager.h>
#include <CEGUIWindowRendererManager.h>
#include <CEGUISchemeManager.h>
#include <CEGUIImagesetManager.h>
#include <CEGUIWindowManager.h>
#include <CEGUIImageset.h>
#include <CEGUIScheme.h>
#include <CEGUIGlobalEventSet.h>
#include <CEGUIDefaultResourceProvider.h>
#include <CEGUIPropertyHelper.h>
#include <CEGUIExceptions.h>

// Utility class for convertion between CEGUI- and wxWidgets types
#include "CEGUIHelper.h"
#include "CEImagesetConfig.h"

// Toolbar icons
#include "bitmaps/new.xpm"
#include "bitmaps/open.xpm"
#include "bitmaps/save.xpm"
#include "bitmaps/cut.xpm"
#include "bitmaps/copy.xpm"
#include "bitmaps/paste.xpm"
#include "bitmaps/delete.xpm"
#include "bitmaps/help.xpm"

// ID codes for the menu items and top-level UI components
const int ID_MENU_BACKGROUND_COLOR = 12000;
const int ID_MENU_RESOURCE_GROUPS = 12001;

const int ID_SPLITTER_FRAME			= 11000;
const int ID_SPLITTER_FRAME_LAYOUT	= 12000;

using namespace CEGUI;

IMPLEMENT_CLASS( EditorFrame, wxDocParentFrame )

//-----------------------------------------------------------------------
BEGIN_EVENT_TABLE( EditorFrame, wxDocParentFrame )
    EVT_MENU( ID_MENU_BACKGROUND_COLOR, EditorFrame::OnChangeBackgroundColour )
    EVT_MENU( ID_MENU_RESOURCE_GROUPS, EditorFrame::OnEditResourceGroups )
END_EVENT_TABLE()

//-----------------------------------------------------------------------
EditorFrame::EditorFrame( wxDocManager* manager, wxFrame* frame, wxWindowID id, const wxString& title,
                          const wxPoint& pos, const wxSize& size, const long type, int backgroundColorR, int backgroundColorG, int backgroundColorB ) :
    wxDocParentFrame( manager, frame, id, title, pos, size, type ),
    m_document( 0 ),
    m_propsPanel( 0 ),
    m_glImagesetCanvas( 0 ),
    m_resGrpsEditor( 0 ),
    m_backgroundColour( backgroundColorR, backgroundColorG, backgroundColorB )
{
    // Add a menu bar, a toolbar and a statusbar
    AttachMenubar(manager);
    AttachToolbar();
    CreateStatusBar( 2 );

    // Create the main dialog
    createContent();

    // pre-create the resource groups editor
    m_resGrpsEditor = new DialogResourceGroups(this);

}

//-----------------------------------------------------------------------
EditorFrame::~EditorFrame()
{
	// save Frame Sash positions
	int sashPos =  m_splitter->GetSashPosition();
	CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_FRAME_VERTICAL_SASH_POS),sashPos);
	sashPos =  m_splitterLeft->GetSashPosition();
	CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_FRAME_HORIZONTAL_SASH_POS),sashPos);
}

//-----------------------------------------------------------------------
void EditorFrame::LoadGroupsFromIniFile()
{
    DefaultResourceProvider* rp =
        static_cast<DefaultResourceProvider*>(
            CEGUI::System::getSingleton().getResourceProvider() );

	wxString groupName, groupDir;
	long dummy;
	CEImagesetConfig::GetInstancePtr()->SetPath(wxT(CONFIG_GROUP_ENTRY));
	bool bCont = CEImagesetConfig::GetInstancePtr()->GetFirstEntry(groupName, dummy);
	while ( bCont ) 
	{
			rp->setResourceGroupDirectory(
				CEGUIHelper::ToCEGUIString( groupName ),
				CEGUIHelper::ToCEGUIString( CEImagesetConfig::GetInstancePtr()->Read(groupName) ) );
		bCont = CEImagesetConfig::GetInstancePtr()->GetNextEntry(groupName, dummy);
	}
	CEImagesetConfig::GetInstancePtr()->SetPath(wxT("/"));


	groupName = CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_DEFAULT_GROUP_IMAGESET));
	if (!groupName.IsEmpty())
		CEGUI::Imageset::setDefaultResourceGroup(CEGUIHelper::ToCEGUIString( groupName ));

	groupName = CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_DEFAULT_GROUP_FONT));
	if (!groupName.IsEmpty())
		CEGUI::Font::setDefaultResourceGroup(CEGUIHelper::ToCEGUIString( groupName ));

	groupName = CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_DEFAULT_GROUP_SCHEME));
	if (!groupName.IsEmpty())
		CEGUI::Scheme::setDefaultResourceGroup(CEGUIHelper::ToCEGUIString( groupName ));

	groupName = CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_DEFAULT_GROUP_LOOKNFEELS));
	if (!groupName.IsEmpty())
		CEGUI::WidgetLookManager::setDefaultResourceGroup(CEGUIHelper::ToCEGUIString( groupName ));

	groupName = CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_DEFAULT_GROUP_LAYOUT));
	if (!groupName.IsEmpty())
		CEGUI::WindowManager::setDefaultResourceGroup(CEGUIHelper::ToCEGUIString( groupName ));

	groupName = CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_DEFAULT_GROUP_SCRIPT));
	if (!groupName.IsEmpty())
		CEGUI::ScriptModule::setDefaultResourceGroup(CEGUIHelper::ToCEGUIString( groupName ));

}

//-----------------------------------------------------------------------
void EditorFrame::AttachMenubar(wxDocManager* manager)
{
    wxMenuBar * menu_bar = new wxMenuBar;

    wxMenu* menu_file = new wxMenu;
    menu_file->Append( wxID_NEW, wxT( "New \tCtrl-N" ), wxT( "Create a new ImageSet." ) );
    menu_file->Append( wxID_OPEN, wxT( "Load \tCtrl-L" ), wxT( "Open an existing ImageSet." ) );
    menu_file->Append( wxID_SAVEAS, wxT( "Save \tCtrl-S" ), wxT( "Save the active ImageSet." ) );
    menu_file->Append( wxID_CLOSE, wxT( "&Close \tCtrl-K" ), wxT( "Close the active ImageSet." ) );
    menu_file->AppendSeparator();
    menu_file->Append( wxID_EXIT, wxT( "Quit\tCtrl-Q" ), wxT( "Quit the application; prompts to save document." ) );
    menu_bar->Append( menu_file, wxT( "File" ) );

    wxMenu* menu_options = new wxMenu;
    menu_options->Append( ID_MENU_BACKGROUND_COLOR, wxT( "Set Background colour..." ), wxT( "choice the colour of the background." ) );
    menu_options->AppendSeparator();
    menu_options->Append( ID_MENU_RESOURCE_GROUPS, wxT( "Resource Groups..." ), wxT( "View and edit currently defined resource groups and directories." ) );
    menu_bar->Append( menu_options, wxT( "Options" ) );

    wxMenu* menu_help = new wxMenu;
    menu_help->Append( wxID_HELP, wxT( "Help..." ), wxT( "" ) );
    menu_help->Append( wxID_ABOUT, wxT( "About..." ), wxT( "Display program information, version number and copyright." ) );
    menu_bar->Append( menu_help, wxT( "?" ) );

    SetMenuBar( menu_bar );

    manager->FileHistoryUseMenu(menu_file);

}

//-----------------------------------------------------------------------
void EditorFrame::AttachToolbar()
{
    // Create toolbar with icons-only
    wxToolBar * toolBar = new wxToolBar( this, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTB_HORIZONTAL ); // |wxNO_BORDER
    //
    toolBar->AddTool( wxID_NEW, wxBitmap ( new_xpm ) );
    toolBar->AddTool( wxID_OPEN, wxBitmap ( open_xpm ) );
    toolBar->AddTool( wxID_SAVE, wxBitmap ( save_xpm ) );
    
	// removed from the old version, as this is never used.
	/*toolBar->AddSeparator();
    toolBar->AddTool( wxID_CUT, wxBitmap ( cut_xpm ) );
    toolBar->AddTool( wxID_COPY, wxBitmap ( copy_xpm ) );
    toolBar->AddTool( wxID_PASTE, wxBitmap ( paste_xpm ) );
    toolBar->AddTool( wxID_CLEAR, wxBitmap ( delete_xpm ) );*/

    toolBar->AddSeparator();
    toolBar->AddTool( wxID_ABOUT, wxBitmap ( help_xpm ) );
    // Don't forget this one!
    toolBar->Realize();
    SetToolBar( toolBar );
}

//-----------------------------------------------------------------------
void EditorFrame::SetDocument( EditorDocument* document )
{
    m_document = document;
}

//-----------------------------------------------------------------------
void EditorFrame::createContent()
{

    m_splitter = new wxSplitterWindow( this, ID_SPLITTER_FRAME, wxDefaultPosition, wxDefaultSize, wxSP_3D);
    m_splitter->SetSashGravity( 0.0 );

	m_splitterLeft = new wxSplitterWindow(m_splitter, ID_SPLITTER_FRAME_LAYOUT,wxDefaultPosition, wxDefaultSize, wxSP_3D);
	m_splitterLeft->SetSashGravity( 0.0 );

    // Create notebook
	m_propertiesNotebook = new wxNotebook(m_splitterLeft, wxID_ANY,wxDefaultPosition, wxSize(300, 200));

	// Add old ImageSetPanel to the notebook
	m_propsPanel = new PropertiesPanel( m_propertiesNotebook, wxDefaultPosition, wxSize( 300, 100 ) );
	m_propsPanel->createContent();
    m_propsPanel->Disable();

	// Add new LayoutPanel to the notebook
	m_layoutPanel = new LayoutPanel( m_propertiesNotebook, wxDefaultPosition, wxSize( 300, 100 ) );
	m_layoutPanel->createContent();
    m_layoutPanel->Disable();

	// Record panels in the notebook
	m_propertiesNotebook->AddPage(m_propsPanel, wxT("ImageSet Properties"), true);
	m_propertiesNotebook->AddPage(m_layoutPanel, wxT("Layout Properties"), false);
	
}

//-----------------------------------------------------------------------
EditorGLCanvas* EditorFrame::AttachCanvas( EditorView* view )
{
    // Create the GLCanvas. This apprauch provides MDI support maybe in the future
    EditorGLCanvas* canvas = new EditorGLCanvas( view, m_splitter, wxDefaultPosition, wxDefaultSize );
    // add the canvas to the frame
    // Tell it about the initial size

	int sashPos = 300;
	CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_FRAME_VERTICAL_SASH_POS),&sashPos);
    m_splitter->SplitVertically( m_splitterLeft, canvas, sashPos );
    canvas->Disable();
    canvas->setBackgroundColour( m_backgroundColour );

    // Copy into member
    m_glImagesetCanvas = canvas;

    return canvas;
}

//-----------------------------------------------------------------------
LayoutGLCanvas* EditorFrame::AttachLayoutCanvas( EditorView* view )
{

    // Create the GLCanvas. This apprauch provides MDI support maybe in the future
	LayoutGLCanvas* layoutCanvas = new LayoutGLCanvas( view, m_splitterLeft, wxDefaultPosition, wxDefaultSize );
	
	// add the canvas to the frame
    // Tell it about the initial size

	int sashPos = 500;
	CEImagesetConfig::GetInstancePtr()->Read(wxT(CONFIG_FRAME_HORIZONTAL_SASH_POS),&sashPos);
    m_splitterLeft->SplitHorizontally( m_propertiesNotebook, layoutCanvas, sashPos);
    layoutCanvas->Disable();
	layoutCanvas->setBackgroundColour( m_backgroundColour );
	layoutCanvas->Render();		// needed to paint the background color at init

    // Copy into member
	m_glLayoutCanvas = layoutCanvas;
	
    return layoutCanvas;
}


//-----------------------------------------------------------------------
void EditorFrame::OnChangeBackgroundColour( wxCommandEvent& WXUNUSED( event ) )
{
    wxColourData colourBackground;
    colourBackground.SetColour( m_backgroundColour );

    // Show colour selector
    wxColourDialog dialog( this, &colourBackground );
    if ( dialog.ShowModal() == wxID_OK )
    {
        m_backgroundColour = dialog.GetColourData().GetColour();

        // set the colour on the canvas
        m_glImagesetCanvas->setBackgroundColour( m_backgroundColour );
		m_glLayoutCanvas->setBackgroundColour( m_backgroundColour );

		m_glImagesetCanvas->Render();
		m_glLayoutCanvas->Render();


		// save the new color in the ini file

		CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_COLOR_RED),m_backgroundColour.Red());
		CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_COLOR_GREEN),m_backgroundColour.Green());
		CEImagesetConfig::GetInstancePtr()->Write(wxT(CONFIG_COLOR_BLUE),m_backgroundColour.Blue());

    }
}

//-----------------------------------------------------------------------
void EditorFrame::OnEditResourceGroups( wxCommandEvent& event )
{
    // the DialogResourceGroups class is basically a self-contained
    // mini-editor for resource groups, so all we have to do is
    // invoke the dialog each time it's asked for.
    m_resGrpsEditor->invoke();
}


